// Separation of Concerns for Binary Projects
// From: https://rust-book.cs.brown.edu/ch12-03-improving-error-handling-and-modularity.html#separation-of-concerns-for-binary-projects
//
// When main starts getting large:
// 1) Split program into main.rs file and lib.rs file and move
//    program's logic to lib.rs
// 2) As long as your command line parsing logic is small, it can
//    remain in main.rs
// 3) When command line parsing logic starts getting complicated,
//    extract it from main.rs and move to lib.rs
//
// Functions that remain in main should be limited to:
// 1) Calling the command line parsing logic with argument values
// 2) Setting up any other configuration
// 3) Calling a run function in lib.rs
// 4) Handling the error if `run` returns an error

use std::env;
use std::process;

use minigrep::Config;

// The modifications for ch13 are added in this version
fn main() {
    // let args: Vec<String> = env::args().collect();
    // dbg!(args);

    // let config = Config::new(&args);

    // unwrap_or_else passes an argument to a closure which is an
    // anonymous function
    // eprintln!() for output to stdout
    // let config = Config::build(&args).unwrap_or_else(|err| {
    let config = Config::build(env::args()).unwrap_or_else(|err| {
        eprintln!("Problem parsing arguments: {err}");
        process::exit(1);
    });

    println!("Searching for {}", config.query);
    println!("In file {}", config.file_path);

    if let Err(e) = minigrep::run(config) {
        eprintln!("Application error: {e}");
        process::exit(1);
    }
}
